/*
  gIR_Volt_test.c
  * Guest Badge IR and Voltage gauge
*/
#include "simpletools.h"
#include "badgetools.h"
#include "datetime.h"

datetime dts = {2000, 1, 1, 0, 0, 0};

datetime dta, dtt;

float calcVt(float batteryVolts, int decayMicros);
float batVolts(float vt);

float RC = 360000.0 * (1.0 / 100000000.0);
float vt, vbat;

int states;
volatile int myLock;

void System_base();
void bLine();


int main()
{
  // Add startup code here.
  badge_setup();
  dt_start(&dts);
  myLock=locknew();
  lockclr(myLock);
  
//  vt = calcVt(4.17, 1471);  //<-Guest Badge specific
  vt = calcVt(4.17, 1453);  // Universal Badge number
  text_size(SMALL);
  oledprint("Any BTN for Menu");
  pause(300);
  cog_run(System_base,128);
 
  while(1)
  {
    // Add main loop code here.

    text_size(SMALL);

      while(!buttons());
      // Lockit
      while(lockset(myLock));
      cursor(0,0);
      bLine();
      cursor(0,1);
      bLine();
      cursor(0,3);
      bLine();
      cursor(0,0);
      oledprint("Receive-P27     ");
      //cursor(0,1);
      //oledprint("Voltage-P26     ");
      lockclr(myLock); 
      // Release the lock.
      while((states = buttons()) == 0);
      switch(states)
      {

        case 0b0000001:   //button P27
          // Lockit for voltage check.
          while(lockset(myLock));
          cursor(0,0);
          bLine();
          cursor(0,1);
          bLine();
          cursor(0, 3);
          text_size(SMALL);
          oledprint("Waiting for IR");

          cursor(0,3);
          oledprint("Ready for IR  ");           // Display ready for IR
          lockclr(myLock);         
          // Release the lock.
    // Wait for IR message
          irclear();                               // Clear ir com buffers
          memset(&dtt, 0, sizeof(datetime));       // Clear their variables

          while(1)
          {           
            irscan("%d%d%d%d%d%d",
            &dtt.y, &dtt.mo, &dtt.d, &dtt.h, &dtt.m, &dtt.s);
            if(dtt.y > 0)
            {
              // Lockit
              while(lockset(myLock));
              dt_copy(&dts, &dtt); //copy new values
              cursor(0,1);
              bLine();
              cursor(0,3);
              bLine();
              cursor(0,0);
              oledprint("Any BTN for Menu");
              lockclr(myLock);
              // Release the lock.
              break;
            }
            //lockclr(myLock);
            // Release the lock.                 
          }
          break;
      /**************************************/
        //case 0b0000010:  //P26
          //cursor(0,0);
          //bLine();
          //cursor(0,1);
          //bLine();
            // Lockit for voltage check.
            //while(lockset(myLock));
            //vbat = batVolts(vt);
            //text_size(SMALL);
            //cursor(0,3);
            //oledprint("%1.2f V", vbat);           
            //pause(3000);
            //lockclr(myLock);
            // Release the lock.
          //cursor(0,0);
          //oledprint("Any BTN for Menu");
          //cursor(0,3);
          //bLine();
         // break;            
      /**************************************/
      }         
  }
 
}


void bLine()
{
  oledprint("                 ");
}  

/* COG for running base utilty. */
void System_base()
{
  while(1)
  {
    while(lockset(myLock));
    cursor(0,7);
    text_size(SMALL);
    oledprint("%02d:%02d:%02d",dts.h, dts.m, dts.s);
    lockclr(myLock);
    pause(50);
    while(lockset(myLock));
    cursor(9,7);
    /*                         month     day   year, last two digits */
    //oledprint("%02d/%02d/%02d",dts.mo, dts.d, dts.y % 100);
    // Voltage
    vbat = batVolts(vt);
    pause(100);
    oledprint("%1.2f V", vbat);
    lockclr(myLock);
  }      
}




float calcVt(float voltsBattery, int microsDecay)
{
  float voltsThreshold, tDecay;
  tDecay = ((float) microsDecay) / 1000000.0;
  voltsThreshold = voltsBattery * (1 - exp(-tDecay / RC));
  return voltsThreshold;
}
  
float batVolts(float voltsThreshold)
{
  float voltsBattery, microsDecay;
  low(0);
  pause(1);
  microsDecay = ((float) rc_time(0, 0)) / 1000000.0;
  voltsBattery = voltsThreshold / (1 - exp(-microsDecay / RC));
  return voltsBattery;
}

